import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_database/firebase_database.dart';
import 'package:flutter/foundation.dart';
import 'package:get/get.dart';
import 'package:wowit/controllers/base_controller.dart';
import 'package:wowit/common.dart';
import 'package:wowit/interfaces/i_watting.dart';
import 'package:wowit/firebase_service.dart';
import 'package:wowit/message.dart';
import 'package:wowit/sql_service.dart';
import 'package:wowit/methods.dart';
import 'package:flutter/material.dart';
import 'package:wowit/widgets/log_widget/log_controller.dart';

abstract class IAuthController {
  Future initSetting(Map<String, List<Map<String, Object>>>? data);
}

class AuthController extends BaseController {
  final FirebaseAuth auth = FirebaseAuth.instance;
  final DatabaseReference db = FirebaseDatabase.instance.ref();
  IAuthController? action;
  final Rx<User?> _firebaseUser = Rx<User?>(null);

  User? get user => _firebaseUser.value;
  bool isInProcess = false;

  updateStatusButton() {
    isInProcess = true;
    update();
  }

  @override
  void onInit() {
    super.onInit();
    _firebaseUser.bindStream(auth.authStateChanges());
  }

  Future initAccountModel() async {
    var uidPhone = Common.getUidPhone();
    var phone = Common.getPhoneNumber();
    if (uidPhone.isEmpty || phone.isEmpty) return;
    Common.uidPhone = uidPhone;
    if (kDebugMode) {
      print('initAccountModel with uid:' + uidPhone);
    }
    if (uidPhone == '') return;
    try {
      await db
          .child('tblAccount/$uidPhone')
          .set({'phone': phone, 'uidPhone': uidPhone}).then((value) async {
        if (kDebugMode) {
          print('==========Thành công==========');
        }
        //create account if not exists
      });
    } catch (e) {
      LogController.setLog(log: e.toString(), titleLog: 'AuthController.initAccountModel');
      if (kDebugMode) {
        print('Error $e');
      }
    }

    await SQLService.createAccountIfNotExists(phone, uidPhone);
  }

  String strVerificationId = '';

  Future<void> signInWithPhone(
      final String phone, StatefulWidget screenOTP, Iwatting iwatting) async {
    updateStatusButton();

    iwatting.showWatting();

    final String phoneNumber = Methods.preparePhone(phone);
    Common.phoneNumberInput = phoneNumber;
    if (phoneNumber == Common.fakeLoginPhone) {
      await Get.to(() => screenOTP);
      return;
    }
    await auth.verifyPhoneNumber(
        phoneNumber: phoneNumber,
        timeout: const Duration(seconds: 0),
        verificationCompleted: (AuthCredential authCredential) async {},
        verificationFailed: (FirebaseAuthException exception) {
          handleExceptionAuth(exception, iwatting);
          return;
        },
        codeSent: (String verificationId, int? forceResendingToken) {
          iwatting.hideWatting();
          if (verificationId == '') {
            handleExceptionAuth(Exception('VerificationId is empty'), iwatting);
            return;
          }
          iwatting.hideWatting();
          //show screen otp
          strVerificationId = verificationId;
          Get.to(() => screenOTP);
        },
        codeAutoRetrievalTimeout: (String verificationId) {
          iwatting.hideWatting();
          strVerificationId = verificationId;
        });
  }

  void verifyOTP(
    String otp,
    Widget homeScreen,
    Iwatting iwatting,
    HandleException handleException,
  ) async {
    updateStatusButton();
    iwatting.showWatting();
    if (kDebugMode) {
      print('===================================');
      print(otp);
      print('${Common.phoneNumberInput} - ${Common.fakeLoginPhone}');
      print('===================================');
    }
    if (otp == '123456' && Common.phoneNumberInput == Common.fakeLoginPhone) {
      if (kDebugMode) {
        print('==========//=========');
        print('ok fake login');
        print('==//===//============');
      }

      await initAccountModel();
      iwatting.hideWatting();
      clearData();
      isInProcess = false;
      await Get.offAll(() => homeScreen);
      return;
    }
    var _credential = PhoneAuthProvider.credential(
      verificationId: strVerificationId,
      smsCode: otp,
    );
    await auth
        .signInWithCredential(_credential)
        .then((UserCredential result) async {
      await initAccountModel();
      iwatting.hideWatting();
      clearData();
      isInProcess = false;
      await Get.offAll(() => homeScreen);
    }).catchError((e) {
      iwatting.hideWatting();

      if (handleException.handleExceptionAuth(e)) {
        handleExceptionAuth(e, iwatting);
      }
      isInProcess = false;
      update();
    });
  }

  void handleExceptionAuth(dynamic e, Iwatting iwatting) {
    iwatting.hideWatting();
    isInProcess = false;
    update();
    if (kDebugMode) {
      print('handleExceptionAuth :' + e.toString());
    }
    if (e.toString().contains('The SMS code has expired')) {
      theSmsOTPTimeOut();
    } else if (e.toString().contains('Timed out waiting for SMS')) {
      theSMSCodeHasExpried();
    } else if (e.toString().contains(
        'We have blocked all requests from this device due to unusual activity')) {
      theBlockRequest();
    } else if (e.toString().contains(
        'The sms verification code used to create the phone auth credential is invalid')) {
      theSMSOTPInValid();
    } else {
      Get.snackbar(
        Message.getMessage('REGISTER_THONG_BAO'),
        e.toString(),
        snackPosition: SnackPosition.TOP,
        duration: const Duration(milliseconds: 10000),
      );
    }
  }

  void theSMSOTPInValid() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_OPT_IN_VALID'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(milliseconds: 10000),
    );
  }

  void theSmsOTPTimeOut() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_TIME_OUT'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(milliseconds: 10000),
    );
  }

  void theBlockRequest() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_BLOCK_REQUEST'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(milliseconds: 10000),
    );
  }

  void theSMSCodeHasExpried() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_HAS_EXPIRED'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(milliseconds: 10000),
    );
  }

  Future<Map<String, List<Map<String, Object>>>?> getLogin() async {
    var uidPhone = Common.getUidPhone();
    var phone = Common.getPhoneNumber();
    if (uidPhone.isEmpty || phone.isEmpty) return null;
    Common.uidPhone = uidPhone;
    if (kDebugMode) {
      print('getLogin uidPhone: ' + uidPhone + ' , phone: ' + phone);
    }
    var rs = await SQLService.execute('sp_getLogin',
        params: {'phone': phone, 'uidPhone': uidPhone});

    var data = Methods.getList(rs, 'info');
    if (data.isNotEmpty) {
      if (Methods.getString(data[0], 'messageId').isNotEmpty) {
        _widgetDialogThongBao();
      }
      if (Methods.getInt(data[0], 'ID') > 0) {
        return rs;
      } else {
        return null;
      }
    }
    return rs;
  }

  void requestPermissionFireBaseMessage(String notifyTokenId) async {
    //REQUEST PERMISSION
    if (await FirebaseService.initializeFirebaseMessaging()) {
      await FirebaseService.initNotifyToken(notifyTokenId);
    }
  }

  void _widgetDialogThongBao({String? keyMessage}) {
    Get.dialog(
      AlertDialog(
        insetPadding: const EdgeInsets.all(50),
        shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(Radius.circular(10.0)),
        ),
        contentPadding: EdgeInsets.zero,
        clipBehavior: Clip.antiAliasWithSaveLayer,
        content: SizedBox(
          height: 100,
          child: Column(
            children: [
              Expanded(
                child: Center(
                  child: Text(
                    Message.getMessage(
                      keyMessage ?? 'TAI_KHOAN_BI_VO_HIEU_HOA',
                    ),
                  ),
                ),
              ),
              const SizedBox(
                width: 8,
              ),
              ElevatedButton(
                onPressed: () => Get.back(),
                style: ButtonStyle(backgroundColor:
                    MaterialStateProperty.resolveWith<Color>((state) {
                  return Common.colorAppbar;
                })),
                child: Text(
                  Message.getMessage('TAI_KHOAN_CLOSE'),
                  style: const TextStyle(color: Colors.white),
                ),
              ),
              const SizedBox(
                width: 8,
              ),
            ],
          ),
        ),
      ),
    );
  }

  @override
  Future<Map<String, List<Map<String, Object>>>?> loadData() async {
    var dt = await getLogin();
    await action?.initSetting(dt);
    return dt;
  }
}
