import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_database/firebase_database.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/foundation.dart';
import 'package:wowit/widgets/log_widget/log_controller.dart';
import 'common.dart';
import 'sql_service.dart';

class FirebaseService {
  FirebaseService._();

  static Future logout() async {
    try {
      var uidPhone = Common.getUidPhone();
      var phone = Common.getPhoneNumber();
      if (uidPhone.isEmpty || phone.isEmpty) return;

      await SQLService.execute('sp_Logout',
          params: {'phone': phone, 'uidPhone': uidPhone});
      await FirebaseAuth.instance.signOut();
    } catch (e) {
      LogController.setLog(log: e.toString(), titleLog: 'FirebaseServerice.logout');
      if (kDebugMode) {
        print('logout: ' + e.toString());
      }
    }
  }

  static Future initNotifyToken(String currentToken) async {
    try {
      var uidPhone = Common.getUidPhone();
      var phone = Common.getPhoneNumber();
      if (uidPhone.isEmpty || phone.isEmpty) return;

      FirebaseMessaging messaging = FirebaseMessaging.instance;
      await messaging.getToken().then((token) async {
        if (currentToken == token) return;
        await SQLService.execute('sp_FirebaseMessaging', params: {
          'phone': phone,
          'uidPhone': uidPhone,
          'notifyToken': token
        });
      });
    } catch (e) {
      LogController.setLog(log: e.toString(), titleLog: 'FirebaseServerice.initNotifyToken');
      if (kDebugMode) {
        print('login; ' + e.toString());
      }
    }
  }

  //REQUEST PERMISSION NOTIFY
  static Future<bool> initializeFirebaseMessaging() async {
    final FirebaseMessaging firebaseMessaging = FirebaseMessaging.instance;
    var settings = await firebaseMessaging.requestPermission(
      alert: true,
      announcement: false,
      badge: true,
      carPlay: false,
      criticalAlert: false,
      provisional: false,
      sound: true,
    );
    if (settings.authorizationStatus == AuthorizationStatus.authorized) {
      if (kDebugMode) {
        print('User granted permission FirebaseMessaging');
      }
      return true;
    } else {
      String log = 'User declined or has not accepted permission FirebaseMessaging';
      LogController.setLog(log: log, titleLog: 'FirebaseServerice.initializeFirebaseMessaging');
      if (kDebugMode) {
        print(log);
      }
      return false;
    }
  }

  static final DatabaseReference _db = FirebaseDatabase.instance.ref();

  static Future<void> getDataRealTime(
      {required String path,
      required Function(Object? value) listen,
      Map<String, Object>? filter}) async {
    DatabaseReference query = _db.child(path);
    if (filter != null && filter.isNotEmpty) {
      filter.forEach((key, value) {
        query.orderByChild(key).equalTo(value);
      });
    }

    query.onValue.listen((event) {
      if (kDebugMode) {
        print('listenOnChange${event.snapshot.value}');
      }
      listen(event.snapshot.value);
    });
  }

  static Future<bool> setDataRealTime(
      {required String path, required Map<String, Object> data}) async {
    try {
      await _db.child(path).set(data);
      return true;
    } catch (e) {
      return false;
    }
  }
}
