import 'package:flutter/foundation.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wowit/common.dart';
import 'package:wowit/sql_service.dart';

import 'methods.dart';

class Message {
  Message._();
  //get message by key
  static String getMessage(String key) {
    if (Common.mapMessageData.isEmpty) Common.setupDefaultData();
    if (!Common.canGetMessage) return key;

    key = key.trim().toUpperCase();
    if (Common.showKeyMessage && Common.supperAdmin) {
      _setMessageBackUp(key, Common.mapMessageData[key]);
      return key;
    }

    if (Common.mapMessageData.containsKey(key)) {
      return Common.mapMessageData[key] ?? key.toUpperCase();
    }

    // not exists -> auto create SQL
    if (kDebugMode && Common.autoCreateMessage) {
      SQLService.execute('sp_SyncMessage', params: {
        'message': key,
        'messageId': key,
        'languageId': 1,
      });
    }

    Common.mapMessageData[key] = key;
    return key;
  }

  static void _setMessageBackUp(String key, String? message) {
    if (!Common.messageIdBackUp.containsKey(key)) {
      if (kDebugMode) {
        print('backUp key:$key, message: $message');
      }
      message = message ?? key.toUpperCase();
      Common.messageIdBackUp[key] = message;
    }
  }

  static void setLanguage(int languageCode) {
    Common.languageId = languageCode;
  }

  static String presLanguage = 'language';
  static String presUseMessageFromSever = 'messageSever';

  static Future<void> getMessageSQL(
    Map<String, String> defaultMessage, {
    List<Map<String, Object>>? dtLanguage,
    VoidCallback? showWatting,
    VoidCallback? hideWatting,
    bool reload = false,
    bool useMessageFromSever = false,
  }) async {
    //cong key kho vao, de dam bao khong gap ky tu user su dung.
    const pattern = '[(=-|';

    SharedPreferences prefs = await SharedPreferences.getInstance();

    int language = Common.languageId;
    Map<String, String> valueMessage = {};
    if (prefs.getStringList(presLanguage) == null || reload) {
      if (reload) {
        showWatting?.call();
      }
      List<String> listLanguage = [];

      if (dtLanguage == null || dtLanguage.isEmpty || reload) {
        var data = await SQLService.execute('getMessage',
            params: {'@languageId': language});
        dtLanguage = Methods.getList(data, 'language');
      }

      if (dtLanguage.length > 200) {
        await SQLService.execute('sp_setHasMapMessage',
            params: {'@uid': Common.uid});

        for (var item in dtLanguage) {
          listLanguage.add(Methods.getString(item, 'messageId') +
              pattern +
              Methods.getString(item, 'message'));
        }
        if (prefs.getStringList(presLanguage) != null) {
          await prefs.remove(presLanguage);
        }
        await prefs.setStringList(presLanguage, listLanguage);
        List<String> list = prefs.getStringList(presLanguage) ?? [];
        for (var item in list) {
          var convert = item.split(pattern);
          if (convert.length > 1) {
            String messageId = convert[0].toString().toUpperCase();
            if (!valueMessage.containsKey(messageId)) {
              valueMessage[messageId] = convert[1];
            }
          }
        }
      } else {
        valueMessage = defaultMessage;
      }
    } else {
      List<String> list = prefs.getStringList(presLanguage) ?? [];
      for (var item in list) {
        var convert = item.split(pattern);
        if (convert.length > 1) {
          String messageId = convert[0].toString().toUpperCase();
          if (!valueMessage.containsKey(messageId)) {
            valueMessage[messageId] = convert[1];
          }
        }
      }
    }
    Common.initWowIT(
      valueMessage,
      Common.url_host,
    );
    if (reload) {
      hideWatting?.call();
    }
  }
}
