import 'package:flutter/material.dart';
import 'package:wowit/message.dart';
import 'dart:io';
import 'package:http/http.dart' as http;
import 'common.dart';

class Methods {
  Methods._();

  static String showDate(DateTime? date) {
    if (date == null) {
      return '';
    }
    return '${date.day}/${date.month}/${date.year}';
  }

  static double round(double d, int r) {
    String s = d.toStringAsFixed(r);
    return double.parse(s);
  }

  static String? removeToneMark(String? str) {
    if (str == null || str.isEmpty) return str;
    str = str.replaceAll(RegExp(r'à|á|ạ|ả|ã|â|ầ|ấ|ậ|ẩ|ẫ|ă|ằ|ắ|ặ|ẳ|ẵ'), 'a');
    str = str.replaceAll(RegExp(r'À|Á|Ạ|Ả|Ã|Â|Ầ|Ấ|Ậ|Ẩ|Ẫ|Ă|Ằ|Ắ|Ặ|Ẳ|Ẵ'), 'A');
    str = str.replaceAll(RegExp(r'è|é|ẹ|ẻ|ẽ|ê|ề|ế|ệ|ể|ễ'), 'e');
    str = str.replaceAll(RegExp(r'È|É|Ẹ|Ẻ|Ẽ|Ê|Ề|Ế|Ệ|Ể|Ễ'), 'E');
    str = str.replaceAll(RegExp(r'ì|í|ị|ỉ|ĩ'), 'i');
    str = str.replaceAll(RegExp(r'Ì|Í|Ị|Ỉ|Ĩ'), 'I');
    str = str.replaceAll(RegExp(r'ò|ó|ọ|ỏ|õ|ô|ồ|ố|ộ|ổ|ỗ|ơ|ờ|ớ|ợ|ở|ỡ'), 'o');
    str = str.replaceAll(RegExp(r'Ò|Ó|Ọ|Ỏ|Õ|Ô|Ồ|Ố|Ộ|Ổ|Ỗ|Ơ|Ờ|Ớ|Ợ|Ở|Ỡ'), 'O');
    str = str.replaceAll(RegExp(r'ù|ú|ụ|ủ|ũ|ư|ừ|ứ|ự|ử|ữ'), 'u');
    str = str.replaceAll(RegExp(r'Ù|Ú|Ụ|Ủ|Ũ|Ư|Ừ|Ứ|Ự|Ử|Ữ'), 'U');
    str = str.replaceAll(RegExp(r'ỳ|ý|ỵ|ỷ|ỹ'), 'y');
    str = str.replaceAll(RegExp(r'Ỳ|Ý|Ỵ|Ỷ|Ỹ'), 'Y');
    str = str.replaceAll(RegExp(r'đ'), 'd');
    str = str.replaceAll(RegExp(r'Đ'), 'D');
    str = str.replaceAll(RegExp(r' '), '');
    return str;
  }

//do khoan cach thoi gian bien truyen vao so voi hien tai
  static String timeAgoString(DateTime date) {
    var secondsInterval = DateTime.now().difference(date).inSeconds;
    if (secondsInterval < 10.0) {
      return Message.getMessage('VAI_GIAY_TRUOC');
    }
    if (secondsInterval < 60.0) {
      return secondsInterval.toString() +
          ' ' +
          Message.getMessage('GIAY_TRUOC');
    }
    double minutes = (secondsInterval / 60.0);
    minutes = round(minutes, 1);
    if (minutes < 60.0) {
      return minutes.round().toString() +
          ' ' +
          Message.getMessage('PHUT_TRUOC');
    }
    var hours = round((minutes / 24.0), 1);
    if (hours < 24.0) {
      return hours.round().toString() + ' ' + Message.getMessage('GIO_TRUOC');
    }
    var days = round((hours / 60.0), 1);
    if (days < 30.0) {
      return days.round().toString() + ' ' + Message.getMessage('NGAY_TRUOC');
    }
    var months = round((days / 30.0), 1);
    if (months < 12.0) {
      return months.round().toString() +
          ' ' +
          Message.getMessage('THANG_TRUOC');
    }
    var years = round((months / 12.0), 1);
    return years.round().toString() + ' ' + Message.getMessage('NAM_TRUOC');
  }

  static Object? getDataHashMap(
    Map? data,
    String key, {
    Object? defaultValue = '',
  }) {
    if (data == null) return defaultValue;
    try {
      dynamic val;
      if (data.containsKey(key)) {
        val = data[key];
        if (val != null && val.toString() != '') {
          return val;
        }
      }
      return defaultValue;
    } catch (e) {
      return defaultValue;
    }
  }

  static List<String> getListString(Map? data, String key) {
    List<String> lst = [];
    if (data == null) return lst;
    try {
      var arrKeys = key.split('.');
      if (arrKeys.length == 2) {
        if (data.containsKey(arrKeys[0])) {
          List<dynamic> lstObj = data[arrKeys[0]] as List<dynamic>;
          if (lstObj.isNotEmpty) {
            for (int index = 0; index < lstObj.length; index++) {
              Map map = lstObj[index];
              lst.add(getString(map, arrKeys[1], defaultValue: ''));
            }
          }
          return lst;
        }
      } else {
        List<dynamic> lstObj = data[key] as List<dynamic>;
        if (lstObj.isNotEmpty) {
          for (int index = 0; index < lstObj.length; index++) {
            lst.add(lstObj[index].toString());
          }
        }
      }
      return lst;
    } catch (e) {
      return lst;
    }
  }

  static List<bool> getListBool(Map? data, String key) {
    List<bool> lst = [];
    if (data == null) return lst;
    try {
      var arrKeys = key.split('.');
      if (arrKeys.length == 2) {
        if (data.containsKey(arrKeys[0])) {
          List<dynamic> lstObj = data[arrKeys[0]] as List<dynamic>;
          if (lstObj.isNotEmpty) {
            for (int index = 0; index < lstObj.length; index++) {
              Map map = lstObj[index];
              lst.add(getBool(map, arrKeys[1], defaultValue: false));
            }
          }
          return lst;
        }
      } else {
        List<dynamic> lstObj = data[key] as List<dynamic>;
        if (lstObj.isNotEmpty) {
          for (int index = 0; index < lstObj.length; index++) {
            lst.add(toBool(lstObj[index].toString()));
          }
        }
      }
      return lst;
    } catch (e) {
      return lst;
    }
  }

  static List<int> getListInt(Map? data, String key) {
    List<int> lst = [];
    if (data == null) return lst;
    try {
      var arrKeys = key.split('.');
      if (arrKeys.length == 2) {
        if (data.containsKey(arrKeys[0])) {
          List<dynamic> lstObj = data[arrKeys[0]] as List<dynamic>;
          if (lstObj.isNotEmpty) {
            for (int index = 0; index < lstObj.length; index++) {
              Map map = lstObj[index];
              lst.add(getInt(map, arrKeys[1], defaultValue: 0));
            }
          }
          return lst;
        }
      } else {
        List<dynamic> lstObj = data[key] as List<dynamic>;
        if (lstObj.isNotEmpty) {
          for (int index = 0; index < lstObj.length; index++) {
            lst.add(toInt(lstObj[index].toString()));
          }
        }
      }
      return lst;
    } catch (e) {
      return lst;
    }
  }

  static List<double> getListDouble(Map? data, String key) {
    List<double> lst = [];
    if (data == null) return lst;
    try {
      var arrKeys = key.split('.');
      if (arrKeys.length == 2) {
        if (data.containsKey(arrKeys[0])) {
          List<dynamic> lstObj = data[arrKeys[0]] as List<dynamic>;
          if (lstObj.isNotEmpty) {
            for (int index = 0; index < lstObj.length; index++) {
              Map map = lstObj[index];
              lst.add(getDouble(map, arrKeys[1], defaultValue: 0));
            }
          }
          return lst;
        }
      } else {
        List<dynamic> lstObj = data[key] as List<dynamic>;
        if (lstObj.isNotEmpty) {
          for (int index = 0; index < lstObj.length; index++) {
            lst.add(toDouble(lstObj[index].toString()));
          }
        }
      }
      return lst;
    } catch (e) {
      return lst;
    }
  }

  static int getInt(Map? data, String key, {int defaultValue = 0}) {
    return toInt(
      getDataHashMap(data, key).toString(),
      defaultValue: defaultValue,
    );
  }

  static double getDouble(Map data, String key, {double defaultValue = 0}) {
    return toDouble(getDataHashMap(data, key).toString(),
        defaultValue: defaultValue);
  }

  static DateTime? getDateTime(
    Map data,
    String key, {
    DateTime? defaultValue,
  }) {
    return toDateTime(
      getDataHashMap(data, key).toString(),
      defaultValue: defaultValue,
    );
  }

  static String getString(Map data, String key, {String defaultValue = ''}) {
    return getDataHashMap(data, key, defaultValue: defaultValue).toString();
  }

  static bool getBool(Map data, String key, {bool defaultValue = false}) {
    return toBool(getDataHashMap(data, key).toString(),
        defaultValue: defaultValue);
  }

  static Color getColor(Map data, String key,
      {Color defaultValue = Colors.red}) {
    try {
      String _color =
          Methods.getString(data, key, defaultValue: ''); //default COLOR _BLUE
      if (_color == '') {
        return defaultValue;
      }
      return HexColor.fromHex(_color);
    } catch (ex) {
      return defaultValue;
    }
  }

  static List<Map<String, Object>> getList(Map data, String key) {
    List<Map<String, Object>> lst = [];
    try {
      if (data.containsKey(key)) {
        List<dynamic> lstObj = data[key] as List<dynamic>;
        if (lstObj.isNotEmpty) {
          for (int index = 0; index < lstObj.length; index++) {
            Map<String, Object> map = lstObj[index];
            lst.add(map);
          }
        }
        return lst;
      }
      return lst;
    } catch (e) {
      return lst;
    }
  }

  static String getURL(String path) {
    if (path.startsWith('http')) {
      return path;
    }
    if (path.startsWith('/')) {
      return Common.url_host + path;
    }
    return Common.url_host + '/' + path;
  }

  static Uri getURI(String path) {
    if (path.startsWith('http')) return Uri.parse(path);
    return Uri.parse(Methods.getURL(path));
  }

  static String getFullURLFileHosting(
      String collection, int documentID, String fileName) {
    String url = Common.url_host;
    if (!url.endsWith('/')) url += '/';
    url += Common.FOLDER_IMAGE_UPLOAD;
    if (!url.endsWith('/')) url += '/';
    url += collection;
    if (!url.endsWith('/')) url += '/';
    if (documentID != -1) url += documentID.toString();
    if (!url.endsWith('/')) url += '/';
    url += fileName;
    return url;
  }

  static double toDouble(String? s, {double defaultValue = 0}) {
    if (s == null) return defaultValue;
    try {
      if (s == '' || s == 'null') {
        return defaultValue;
      }
      double d = double.parse(s);
      return d;
    } catch (e) {
      return defaultValue;
    }
  }

  static int toInt(String? s, {bool isFloor = true, int defaultValue = 0}) {
    if (s == null) return defaultValue;
    try {
      if (s == '' || s == 'null') {
        return defaultValue;
      }
      double d = toDouble(s);
      if (isFloor) {
        return d.floor();
      } else {
        return int.parse(s);
      }
      // return i;
    } catch (e) {
      return defaultValue;
    }
  }

  static bool toBool(String? s, {bool defaultValue = false}) {
    if (s == null) return false;
    try {
      return s == '1' || s.toLowerCase() == 'true';
    } catch (e) {
      return defaultValue;
    }
  }

  static DateTime? toDateTime(
    String? s, {
    DateTime? defaultValue,
  }) {
    if (s == null) return defaultValue;
    try {
      if (s == 'null' || s == '' || s.length < 8) {
        return defaultValue;
      }
      return DateTime.parse(s);
    } catch (e) {
      return defaultValue;
    }
  }

//doc tien thanh chu
//lienHe : neu so tien = 0 -> tra ve 'Lien He'
  static String docTienThanhChu(double soTien,
      {int roundDecimal = 2, bool lienHe = true, bool rutGon = false}) {
    int i;

    if (soTien > 8999999999999999) {
      return soTien.toString();
    }
    if (soTien <= 0 && lienHe) {
      return Message.getMessage('GIA_LIEN_HE');
    }

    var tienSo = Common.tienSo;
    var tienDonVi = Common.getTienDonVi(rutGon);
    for (i = 0; i < tienSo.length; i++) {
      if (soTien / tienSo[i] > double.maxFinite ||
          soTien / tienSo[i] < double.minPositive) {
        return soTien.toString();
      }

      if (soTien >= tienSo[i]) {
        if (soTien % tienSo[i] > 0) {
          if (roundDecimal > 0) {
            var r = round((soTien / tienSo[i]), roundDecimal);
            if (r == 0 && lienHe) {
              return Message.getMessage('GIA_LIEN_HE');
            }
            return r.toString() + ' ' + tienDonVi[i];
          } else {
            double lastValue = (soTien / tienSo[i]);

            if (lastValue == 0 && lienHe) {
              return Message.getMessage('GIA_LIEN_HE');
            }
            return (lastValue).toString() + ' ' + (tienDonVi[i]);
          }
        } else {
          int iSoTien = toInt((soTien / tienSo[i]).toString());
          if (iSoTien == 0 && lienHe) {
            return Message.getMessage('GIA_LIEN_HE');
          }
          return iSoTien.toString() + ' ' + tienDonVi[i];
        }
      }
    }
    if (soTien == 0 && lienHe) {
      return Message.getMessage('GIA_LIEN_HE');
    }
    return soTien.toString();
  }

  static String getChanelId(int uidFrom, int uidTo) {
    return (uidFrom > uidTo)
        ? (uidFrom.toString() + '_' + uidTo.toString())
        : (uidTo.toString() + '_' + uidFrom.toString());
  }

  static String preparePhone(String phone) {
    String phoneNumber;

    if (phone.startsWith('0')) {
      phoneNumber = '+84' + phone.substring(1);
    } else if (phone.startsWith('84')) {
      phoneNumber = '+' + phone;
    } else {
      phoneNumber = phone;
    }
    return phoneNumber;
  }

  static Future<String> uploadFileParent(
      File file, String tableName, int id, String fieldName,
      {String fileName = ''}) async {
    //create multipart request for POST or PATCH method
    if (fileName == '') {
      fileName = getFileName(file.path);
    }
    var request = http.MultipartRequest('POST',
        Uri.parse(Common.url_host + '/File/UploadImageFieldNameParent'));
    //add text fields
    request.fields['tableName'] = tableName;
    request.fields['id'] = id.toString();
    request.fields['fieldName'] = fieldName;
    request.fields['fileName'] = fileName;
    //create multipart using filepath, string or bytes
    var pic = await http.MultipartFile.fromPath('file', file.path);
    //add multipart to request
    request.files.add(pic);
    var response = await request.send();

    //Get the response from the server
    var responseData = await response.stream.toBytes();
    var responseString = String.fromCharCodes(responseData);
    return responseString;
  }

  static Future<String> uploadFileParentChild(
    File file,
    String tableName,
    int id,
    String fieldName,
    String parentTableName,
    int parentId, {
    String fileName = '',
  }) async {
    //create multipart request for POST or PATCH method
    if (fileName == '') {
      fileName = getFileName(file.path);
    }
    var request = http.MultipartRequest(
        'POST', Uri.parse(Common.url_host + '/File/UploadImageFieldNameChild'));
    //add text fields
    request.fields['tableName'] = tableName;
    request.fields['id'] = id.toString();
    request.fields['fieldName'] = fieldName;
    request.fields['parentTableName'] = parentTableName;
    request.fields['parentId'] = parentId.toString();
    request.fields['fileName'] = fileName;
    //create multipart using filepath, string or bytes
    var pic = await http.MultipartFile.fromPath('file', file.path);
    //add multipart to request
    request.files.add(pic);
    var response = await request.send();

    //Get the response from the server
    var responseData = await response.stream.toBytes();
    var responseString = String.fromCharCodes(responseData);
    return responseString;
  }

  static String getFileName(String fullPath) {
    String fileName = fullPath;
    if (fullPath.contains('/')) {
      var arr = fullPath.split('/');
      fileName = arr[arr.length - 1];
    }
    return fileName;
  }

  static String getFilePath(String fullPath) {
    String filePath = fullPath;
    if (fullPath.contains('\\')) {
      filePath = fullPath.split('\\').last;
    }
    return filePath;
  }

  static int getFileId(String fullPath) {
    int fileId = 0;
    if (fullPath.contains('/')) {
      var arr = fullPath.split('/');
      String s = '0';
      if (arr.length > 1) {
        s = arr[arr.length - 2];
      }
      fileId = Methods.toInt(s);
    }
    return fileId;
  }

  static bool validPhoneNumber(String? phone, {String countryCode = 'vn'}) {
    if (phone == null) return false;
    String checkPhone = phone;
    if (checkPhone.startsWith('+84')) {
      checkPhone = checkPhone.replaceAll('+84', '0');
    }
    if (checkPhone.startsWith('84')) {
      checkPhone = checkPhone.replaceAll('84', '0');
    }

    if (checkPhone.length != 10 && countryCode != 'vn') return false;
    bool found = false;
    for (CoutryPhoneNumber s in Common.listCoutryPhoneNumber) {
      if (s.countryCode == countryCode) {
        for (String ss in s.listStartPhoneHeader) {
          if (checkPhone.startsWith(ss)) {
            found = true;
            break;
          }
        }
      }
    }
    if (!found) return false;

    var pattern = '[0-9]{10}\$';

    var regex = RegExp(pattern);
    if (regex.hasMatch(checkPhone)) {
      return true;
    } else {
      return false;
    }
  }

  static List<Map<String, Object>> getListChildTable(
      Map<String, List<Map<String, Object>>> data, int idItem, String key,
      {String keyRef = 'refID'}) {
    List<Map<String, Object>> mapResult = [];
    for (var item in Methods.getList(data, key)) {
      if (Methods.getInt(item, keyRef) == idItem) {
        mapResult.add(item);
      }
    }
    return mapResult;
  }

  static String getValueListHashMap(
    List<Map<String, Object>> data,
    String value,
    String fieldNameCompare,
    String fileNameReturn, {
    Object? defaultValue,
  }) {
    if (data.isEmpty) return defaultValue.toString();
    for (var item in data) {
      if (Methods.getDataHashMap(item, fieldNameCompare, defaultValue: null) ==
          value) {
        return Methods.getDataHashMap(
          item,
          fileNameReturn,
          defaultValue: null,
        ).toString();
      }
    }
    return defaultValue.toString();
  }

  static List<File> getListFile(dynamic data) {
    List<File> files = [];
    if (data is File) {
      files.add(data);
    } else if (data is Iterable) {
      for (var e in data) {
        files.addAll(getListFile(e));
      }
    } else if (data is Map) {
      for (var e in data.values) {
        files.addAll(getListFile(e));
      }
    }
    return files;
  }
}

extension HexColor on Color {
  /// String is in the format "aabbcc" or "ffaabbcc" with an optional leading "#".
  static Color fromHex(String hexString) {
    final buffer = StringBuffer();
    if (hexString.length == 6 || hexString.length == 7) buffer.write('ff');
    buffer.write(hexString.replaceFirst('#', ''));
    return Color(int.parse(buffer.toString(), radix: 16));
  }

  /// Prefixes a hash sign if [leadingHashSign] is set to `true` (default is `true`).
  String toHex({bool leadingHashSign = true}) => '${leadingHashSign ? '#' : ''}'
      '${alpha.toRadixString(16).padLeft(2, '0')}'
      '${red.toRadixString(16).padLeft(2, '0')}'
      '${green.toRadixString(16).padLeft(2, '0')}'
      '${blue.toRadixString(16).padLeft(2, '0')}';
}
