import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/controllers/base_controller.dart';
import 'package:wowit/widgets/base_widget.dart';

class BaseWidgetBuilder<T extends BaseController> extends BaseWidget<T> {
  const BaseWidgetBuilder({
    super.key,
    super.global,
    super.isComponent,
    super.tag,
    required T controller,
    required this.builder,
    this.skeletonBuilder,
    Function(GetBuilderState<T> state)? initState,
    Function(GetBuilderState<T> state)? dispose,
    Function(GetBuilderState<T> state)? didChangeDependencies,
    Function(GetBuilder oldWidget, GetBuilderState<T> state)? didUpdateWidget,
    String Function(DiagnosticLevel minLevel)? toString,
  })  : _initState = initState,
        _dispose = dispose,
        _didChangeDependencies = didChangeDependencies,
        _didUpdateWidget = didUpdateWidget,
        _toString = toString,
        super(controller);

  final Widget Function(BuildContext context,
      Map<String, List<Map<String, Object>>> data, T controller) builder;
  final Function(GetBuilderState<T> state)? _initState;
  final Function(GetBuilderState<T> state)? _dispose;
  final Function(GetBuilderState<T> state)? _didChangeDependencies;
  final Function(GetBuilder oldWidget, GetBuilderState<T> state)?
      _didUpdateWidget;
  final Widget Function(BuildContext context, T controller)? skeletonBuilder;
  final String Function(DiagnosticLevel minLevel)? _toString;

  @override
  void initState(GetBuilderState<T> state) {
    super.initState(state);
    _initState?.call(state);
  }

  @override
  void dispose(GetBuilderState<T> state) {
    super.dispose(state);
    _dispose?.call(state);
  }

  @override
  void didUpdateWidget(GetBuilder oldWidget, GetBuilderState<T> state) {
    super.didUpdateWidget(oldWidget, state);
    _didUpdateWidget?.call(oldWidget, state);
  }

  @override
  void didChangeDependencies(GetBuilderState<T> state) {
    super.didChangeDependencies(state);
    _didChangeDependencies?.call(state);
  }

  @override
  String toString({DiagnosticLevel minLevel = DiagnosticLevel.info}) =>
      _toString?.call(minLevel) ?? super.toString(minLevel: minLevel);

  @override
  Widget getMainView(
    BuildContext context,
    Map<String, List<Map<String, Object>>> data,
    T controller,
  ) {
    return builder.call(context, data, controller);
  }

  @override
  Widget getSkeletonView(
    BuildContext context,
    T controller,
  ) {
    return skeletonBuilder?.call(context, controller) ??
        super.getSkeletonView(context, controller);
  }
}
