part of edit_screen;

class EditorController {
  EditorController(
    Map<String, Object>? dataEntity,
    this.onSaveProcedure,
    this.editorItems,
  ) {
    _addDefaultValues();
    initalValue(dataEntity);
  }

  final List<EditorItem> editorItems;
  final Map<String, Object?> dataEntity = {};
  final String onSaveProcedure;
  String? _messageInvalidDialog;
  bool _isDirtyData = false;
  final Rx<bool?> _isConfirm = false.obs;
  final Rx<bool?> _isWaiting = false.obs;
  final Rx<bool?> _succeed = false.obs;
  final Rx<bool?> _isShowInValidDialog = false.obs;
  VoidCallback? onSuccess;
  static late Map<String, Object> mapDataReturn;

  void initalValue(Map<String, Object>? initialMapValue) {
    if (initialMapValue != null) {
      dataEntity.addEntries(initialMapValue.entries);
    }
  }

  void onValueChanged(String fieldName, Object value) {
    Object? _value = value;
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem?.type == EEditorType.int && (value is String)) {
      _value = int.tryParse(value);
    } else if (editorItem?.type == EEditorType.double && (value is String)) {
      _value = double.tryParse(value);
    } else if (editorItem?.type == EEditorType.money && (value is String)) {
      _value = double.tryParse(value.replaceAll(RegExp(','), ''));
    }

    dataEntity.update(
      fieldName,
      (_) => _value,
      ifAbsent: () => _value,
    );
    _isDirtyData = true;
  }

  void showInvalidDialog({required String message}) {
    _messageInvalidDialog = message;
    _isShowInValidDialog.value = true;
  }

  String getHintText(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem == null) {
      return hintMessage;
    }
    if (editorItem.hint != null && editorItem.hint!.isNotEmpty) {
      return editorItem.hint!;
    }
    return hintMessage;
  }

  String getInvalidText(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem == null) return invalidMessage;
    if (editorItem.invalidMessage != null &&
        editorItem.invalidMessage!.isNotEmpty) {
      return editorItem.invalidMessage!;
    }
    return invalidMessage;
  }

  String getText(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);

    if (dataEntity[fieldName] == null) return '';

    if (editorItem?.type == EEditorType.money) {
      return PriceInputFormatter.format(
        (dataEntity[fieldName] as double).toInt().toString(),
      );
    }

    return dataEntity[fieldName].toString();
  }

  String? getValidator(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem == null) return null;
    String? _checkMinMax = checkMinMax(editorItem);
    if (_checkMinMax != null) {
      return _checkMinMax;
    }

    bool isRequired = editorItem.isRequired;

    String title = editorItem.title ?? '';
    if (title.isNotEmpty && !isRequired) {
      isRequired = title.startsWith('*') || title.endsWith('*');
    }

    if (!isRequired) return null;

    if (editorItem.validator != null) {
      return editorItem.validator?.call(dataEntity[fieldName]);
    }

    switch (editorItem.type) {
      case EEditorType.image:
        return _validateImage(fieldName)
            ? null
            : editorItem.invalidMessage ?? invalidMessage;
      case EEditorType.string:
        return _validateString(fieldName)
            ? null
            : editorItem.invalidMessage ?? invalidMessage;
      case EEditorType.phone:
        return Methods.validPhoneNumber(dataEntity[fieldName] as String)
            ? null
            : editorItem.invalidMessage ?? invalidPhone;
      case EEditorType.email:
        return (dataEntity[fieldName] as String).isEmail
            ? null
            : _getInvalidEmailValidator(editorItem);
      case EEditorType.int:
        return dataEntity[fieldName] != null
            ? null
            : editorItem.invalidMessage ?? invalidMessage;
      case EEditorType.double:
        return dataEntity[fieldName] != null
            ? null
            : editorItem.invalidMessage ?? invalidMessage;
      case EEditorType.money:
        return _validateMoney(fieldName)
            ? null
            : editorItem.invalidMessage ?? invalidMessage;
      case EEditorType.datetime:
        return _validateDateTime(fieldName)
            ? null
            : editorItem.invalidMessage ?? invalidMessage;
      default:
        return dataEntity[fieldName] != null
            ? null
            : editorItem.invalidMessage ?? invalidMessage;
    }
  }

  TextInputType? getTextInputType(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem == null) return null;
    switch (editorItem.type) {
      case EEditorType.int:
      case EEditorType.money:
        return TextInputType.number;
      case EEditorType.double:
        return const TextInputType.numberWithOptions(decimal: true);
      case EEditorType.phone:
        return TextInputType.phone;
      case EEditorType.email:
        return TextInputType.emailAddress;
      case EEditorType.datetime:
        return TextInputType.datetime;
      case EEditorType.name:
      case EEditorType.string:
      case EEditorType.image:
      default:
        return null;
    }
  }

  List<TextInputFormatter> getInputFormatters(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);

    if (editorItem == null) return <TextInputFormatter>[];

    if (editorItem.type == EEditorType.money) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
        PriceInputFormatter(),
      ];
    }

    if (editorItem.type == EEditorType.int ||
        editorItem.type == EEditorType.phone) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
      ];
    }

    if (editorItem.type == EEditorType.name) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.allow(
          RegExp(
              '[a-zA-Z_ÀÁÂÃÈÉÊÌÍÒÓÔÕÙÚĂĐĨŨƠàáâãèéêìíòóôõùúăđĩũơƯĂẠẢẤẦẨẪẬẮẰẲẴẶẸẺẼỀỀỂưăạảấầẩẫậắằẳẵặẹẻẽềềểỄỆỈỊỌỎỐỒỔỖỘỚỜỞỠỢỤỦỨỪễếệỉịọỏốồổỗộớờởỡợụủứừỬỮỰỲỴÝỶỸửữựỳỵỷỹ  \t]'),
        ),
      ];
    }

    return <TextInputFormatter>[];
  }

  String _getInvalidEmailValidator(EditorItem item) {
    String email = Methods.getString(dataEntity, item.fieldName);
    if (email.isEmpty) {
      return item.invalidMessage ?? invalidMessage;
    } else {
      return Message.getMessage('EDITOR_EMAIL_INVALID');
    }
  }

  String? getTitleByFieldName(String fieldName) {
    return getEditorItemByFieldName(fieldName)?.title;
  }

  bool _checkIsImagesKey(String fieldName) {
    final editorItem = getEditorItemByFieldName(fieldName);
    if (editorItem?.type == EEditorType.image) {
      return true;
    }
    return false;
  }

// #region Validate Data
  bool validate({int step = 1, bool isShowInvalidDialog = true}) {
    if (!_checkValidData(step)) {
      if (isShowInvalidDialog == true) {
        _isShowInValidDialog.value = true;
      }
      return false;
    } else {
      return true;
    }
  }

  bool _checkValidData(int step) {
    for (final editorItem in editorItems) {
      if ((editorItem.step != step) || !editorItem.isRequired) {
        continue;
      }
      _messageInvalidDialog = invalidMessage;
      return false;
    }
    return true;
  }

  bool _validateString(String fieldName) {
    var value = dataEntity[fieldName];
    if (value == null) return false;
    if (value is String) {
      return value.isNotEmpty;
    }
    return true;
  }

  bool _validateMoney(String fieldName) {
    var value = dataEntity[fieldName];
    if (value == null) return false;
    if (value is num) {
      return value > 0;
    }
    return false;
  }

  bool _validateDateTime(String fieldName) {
    var value = dataEntity[fieldName];
    if (value == null) return false;
    if (value is DateTime) return true;
    if (value is String) {
      return DateTime.tryParse(value) != null;
    }
    return false;
  }

  void _resetAfterDone() {
    _isWaiting.value = false;
    _succeed.value = null;
    _isDirtyData = false;
    _addDefaultValues();
  }

  void _resetAfterFail() {
    _isWaiting.value = false;
    _succeed.value = false;
    _isDirtyData = true;
  }

  void _addDefaultValues() {
    for (var item in editorItems) {
      dataEntity.addAll({item.fieldName: item.defaultValue});
    }
  }

  EditorItem? getEditorItemByFieldName(String fieldName) {
    EditorItem? editorItem = editorItems.firstWhereOrNull(
      (element) => element.fieldName == fieldName,
    );
    return editorItem;
  }

  bool _validateImage(String fieldName) {
    List<File> images = Methods.getListFile(dataEntity[fieldName]);
    if (images.isEmpty) return false;
    return true;
  }

  String get invalidMessage => Message.getMessage('EDITOR_INVALID_MESSAGE');
  String get invalidPhone => Message.getMessage('EDITOR_INVALID_PHONE');
  String get hintMessage => Message.getMessage('EDITOR_HINT_MESSAGE');
}

extension MinMax on EditorController {
  String? checkMinMax(EditorItem item) {
    switch (item.type) {
      case EEditorType.image:
        int length = Methods.getListFile(dataEntity[item.fieldName]).length;
        return _checkMinMaxInt(
          length,
          min: item.min?.toInt(),
          max: item.max?.toInt(),
          invalidMin: item.invalidMin ?? item.invalidMessage,
          invalidMax: item.invalidMax ?? item.invalidMessage,
        );
      case EEditorType.int:
        int value = Methods.getInt(dataEntity, item.fieldName);
        return _checkMinMaxInt(
          value,
          min: item.min?.toInt(),
          max: item.max?.toInt(),
          invalidMin: item.invalidMin ?? item.invalidMessage,
          invalidMax: item.invalidMax ?? item.invalidMessage,
        );
      case EEditorType.double:
      case EEditorType.money:
        double value = Methods.getDouble(dataEntity, item.fieldName);
        return _checkMinMaxDouble(
          value,
          min: item.min,
          max: item.max,
          invalidMin: item.invalidMin ?? item.invalidMessage,
          invalidMax: item.invalidMax ?? item.invalidMessage,
        );
      case EEditorType.name:
      case EEditorType.string:
      case EEditorType.email:
      case EEditorType.phone:
        String text = Methods.getString(dataEntity, item.fieldName);

        return _checkMinMaxString(
          text,
          min: item.min?.toInt(),
          max: item.max?.toInt(),
          invalidMin: item.invalidMin ?? item.invalidMessage,
          invalidMax: item.invalidMax ?? item.invalidMessage,
        );
      default:
        return null;
    }
  }

  String? _checkMinMaxString(
    String text, {
    int? min,
    int? max,
    String? invalidMin,
    String? invalidMax,
  }) {
    if (min != null && text.length < min) {
      return invalidMin ?? invalidMessage;
    } else if (max != null && text.length > max) {
      return invalidMax ?? invalidMessage;
    }
    return null;
  }

  String? _checkMinMaxInt(
    int value, {
    int? min,
    int? max,
    String? invalidMin,
    String? invalidMax,
  }) {
    if (min != null && value < min) {
      return invalidMin ?? invalidMessage;
    } else if (max != null && value > max) {
      return invalidMax ?? invalidMessage;
    }
    return null;
  }

  String? _checkMinMaxDouble(
    double value, {
    double? min,
    double? max,
    String? invalidMin,
    String? invalidMax,
  }) {
    if (min != null && value < min) {
      return invalidMin ?? invalidMessage;
    } else if (max != null && value > max) {
      return invalidMax ?? invalidMessage;
    }
    return null;
  }
}
