import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/controllers/base_controller.dart';
import 'package:wowit/widgets/base_widget.dart';

import 'view/editor_screen.dart';

abstract class EditorWidget<T extends BaseController> extends BaseWidget<T> {
  EditorWidget(
    super.baseController, {
    super.key,
    required List<EditorItem> editorItems,
    Map<String, Object>? initialDataEntity,
    required String onSaveProcedure,
  })  : title = null,
        isShowAppBar = false,
        actionAppbar = null,
        color = null,
        _editorController =
            EditorController(initialDataEntity, onSaveProcedure, editorItems);

  EditorWidget.screen(
    super.baseController, {
    super.key,
    required this.title,
    this.color,
    required List<EditorItem> editorItems,
    Map<String, Object>? initialDataEntity,
    required String onSaveProcedure,
    this.actionAppbar,
  })  : isShowAppBar = true,
        _editorController =
            EditorController(initialDataEntity, onSaveProcedure, editorItems);

  final String? title;
  final Color? color;
  final bool isShowAppBar;
  final EditorController _editorController;
  EditorController get editorController => _editorController;
  final List<Widget>? actionAppbar;

  Widget getEditorControlView(BuildContext buildContext,
      EditorController editorController, BaseController baseController);

  void onBack() {
    Get.back();
  }

  Widget _getEditorView(BuildContext context, BaseController baseController) {
    if (isShowAppBar) {
      return EditorScreen(
        editorControlView: getEditorControlView(
          context,
          _editorController,
          baseController,
        ),
        editorController: _editorController,
        title: title ?? '',
        onBack: onBack,
        color: color,
        actionAppbar: actionAppbar,
      );
    } else {
      return getEditorControlView(context, editorController, baseController);
    }
  }

  @override
  Widget getMainView(BuildContext context,
      Map<String, List<Map<String, Object>>> data, BaseController controller) {
    return _getEditorView(context, controller);
  }

  bool validate({
    int step = 1,
    bool isShowInvalidDialog = false,
  }) {
    return _editorController.validate(
      step: step,
      isShowInvalidDialog: isShowInvalidDialog,
    );
  }

  void onValueChanged(String fieldName, Object value) {
    _editorController.onValueChanged(fieldName, value);
  }

  void onSave() {
    _editorController.save();
  }
}

class EditorItem {
  const EditorItem({
    required this.fieldName,
    required this.type,
    this.step = 1,
    this.title,
    this.validator,
    this.isRequired = false,
    this.uploadImageInfo,
    this.defaultValue,
    this.hint,
    this.invalidMessage,
    this.min,
    this.max,
    this.invalidMin,
    this.invalidMax,
  });

  final String fieldName;
  final bool isRequired;
  final Object? defaultValue;
  final int step;
  final EEditorType type;
  final String? title;
  final String? hint;
  final String? invalidMessage;
  final String? invalidMin;
  final String? invalidMax;
  final String? Function(Object? value)? validator;
  final UploadImageInfo? uploadImageInfo;

  /// Giới hạn tối thiểu
  ///
  /// Nếu là dạng chữ (String) thì min = số lượng chữ tối thiểu
  ///
  /// Nếu là dạng số (int, double) thì min = số bé nhất
  ///
  /// Nếu null nghĩa là ko ràng buộc
  final double? min;

  /// Giới hạn tối đa
  ///
  /// Nếu là dạng chữ (String) thì max = số lượng chữ tối đa
  ///
  /// Nếu là dạng số (int, double) thì max = số lớn nhất
  ///
  /// Nếu null nghĩa là ko ràng buộc
  final double? max;
}

enum EEditorType {
  image,
  int,
  double,
  string,
  money,
  phone,
  email,
  datetime,
  name,
  bool,
}

class UploadImageInfo {
  UploadImageInfo({
    required this.tableName,
    required this.fieldName,
    this.parentTableName,
    required this.responseIdsName,
  });

  final String tableName;
  final String fieldName;
  final String? parentTableName;
  final String responseIdsName;
}
