library edit_screen;

import 'dart:async';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';
import 'package:wowit/widgets/editor_widget/view/price_format.dart';
import 'package:wowit/common.dart';
import 'package:wowit/message.dart';
import 'package:wowit/methods.dart';
import 'package:wowit/sql_service.dart';

import '../editor_widget.dart';
import 'invalid_data_dialog.dart';

part '../editor_controller.dart';
part '../view/confirm_bottom_sheet.dart';
part '../view/loading_editor_dialog.dart';
part '../editor_upload.dart';

class EditorScreen extends StatefulWidget {
  const EditorScreen({
    super.key,
    required this.title,
    required this.editorControlView,
    this.color,
    required this.editorController,
    this.onBack,
    this.actionAppbar,
  });

  final String title;
  final Widget editorControlView;
  final EditorController editorController;
  final VoidCallback? onBack;
  final List<Widget>? actionAppbar;

  //Custom
  final Color? color;
  @override
  _EditorScreenState createState() => _EditorScreenState();
}

class _EditorScreenState extends State<EditorScreen> {
  @override
  void initState() {
    if (kDebugMode) {
      print('initState EditorScreen');
    }
    _listenDataEditorController();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: PreferredSize(
        preferredSize: const Size.fromHeight(50.0),
        child: AppBar(
          automaticallyImplyLeading: false,
          centerTitle: true,
          backgroundColor: Colors.white,
          elevation: 0.0,
          leading: IconButton(
            onPressed: () {
              if (widget.onBack != null) {
                widget.onBack!.call();
              } else {
                Get.back();
              }
            },
            icon: Icon(
              Icons.arrow_back_ios_outlined,
              color: widget.color ?? Common.colorAppbar,
            ),
          ),
          title: Text(
            widget.title,
            style: TextStyle(
              fontWeight: FontWeight.bold,
              fontSize: 20.0,
              color: widget.color ?? Common.colorAppbar,
            ),
          ),
          actions: widget.actionAppbar,
        ),
      ),
      body: widget.editorControlView,
    );
  }

  Future<void> _listenDataEditorController() async {
    widget.editorController._isWaiting.listen((isWaiting) async {
      if (isWaiting != null && isWaiting) {
        await Get.generalDialog(
          barrierDismissible: false,
          pageBuilder: (context, animation, secondaryAnimation) {
            return LoadingEditorDialog(
              editorController: widget.editorController,
            );
          },
        );
      }
      if (widget.editorController._succeed.value != null) {
        if (widget.editorController._succeed.value!) {
          widget.editorController._resetAfterDone();
        } else {
          widget.editorController._resetAfterFail();
        }
      }
    });

    widget.editorController._isShowInValidDialog.listen((isShowNotValidDialog) {
      if (isShowNotValidDialog != null && isShowNotValidDialog) {
        widget.editorController._isShowInValidDialog.value = false;
        Get.dialog(
          InvalidDataDialog(
            title: widget.editorController._messageInvalidDialog,
          ),
        );
        widget.editorController._messageInvalidDialog = null;
      }
    });

    widget.editorController._isConfirm.listen(
      (isSaving) {
        if (isSaving != null && isSaving) {
          showModalBottomSheet<void>(
            backgroundColor: Colors.grey[300],
            shape: const RoundedRectangleBorder(
              borderRadius: BorderRadius.only(
                topLeft: Radius.circular(16),
                topRight: Radius.circular(16),
              ),
            ),
            context: context,
            builder: (_) => SizedBox(
              width: Get.width - 100,
              child: ConfirmBottomSheet(widget.editorController),
            ),
          );
        }
      },
    );
  }
}
