import 'package:flutter/rendering.dart';
import 'package:wowit/controllers/searched_controller.dart';
import 'package:wowit/widgets/filter_widget/component/seached_list.dart';
import 'package:wowit/widgets/filter_widget/filter_widget.dart';
import 'package:wowit/message.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class SearchScreen extends StatefulWidget {
  const SearchScreen({
    super.key,
    required this.searchedListModel,
    required this.filterIcon,
    required this.fieldNameSearching,
    this.loadSearchedData,
    this.inputDecoration
  });

  final SearchedDataLoader? loadSearchedData;
  final String fieldNameSearching;
  final Widget filterIcon;
  final InputDecoration? inputDecoration;

  final SearchedListModel searchedListModel;

  @override
  _SearchScreenState createState() => _SearchScreenState();
}

class _SearchScreenState extends State<SearchScreen> {
  final keywordController = TextEditingController();
  final ScrollController scrollController = ScrollController();
  late final SearchedList wSearchedList;

  void searchHome(String query) {
    wSearchedList.search(query);
  }

  @override
  void initState() {
    final filterItems = <FilterItem>[];

    filterItems.addAll(widget.searchedListModel.filterItems);

    filterItems.add(
      FilterItem(
        fieldName: widget.fieldNameSearching,
        filterMode: EFilterMode.insensitiveAndUnsignedContain,
      ),
    );

    wSearchedList = SearchedList(
      searchedListModel: widget.searchedListModel,
      fieldNameSearching: widget.fieldNameSearching,
      searchedDataLoader: widget.loadSearchedData,
    );

    wSearchedList.scrollController = scrollController;

    scrollController.addListener(() {
      if (scrollController.position.extentAfter < 200 &&
          scrollController.position.userScrollDirection ==
              ScrollDirection.reverse) {
        wSearchedList.loadMore.call();
      }
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    if (wSearchedList.getValueConditionByFieldName(widget.fieldNameSearching) ==
        null) {
      wSearchedList.search(keywordController.text);
    }
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: _buildAppBar(context),
      body: wSearchedList,
    );
  }

  AppBar _buildAppBar(BuildContext context) {
    return AppBar(
      leading: IconButton(
        onPressed: () {
          FocusScope.of(context).requestFocus(FocusNode());
          Get.back();
        },
        icon: const Icon(
          Icons.arrow_back_ios_outlined,
          color: Color(0xff2a367e),
        ),
        splashRadius: 24,
      ),
      titleSpacing: 0.0,
      elevation: 0.5,
      title: _SearchTextField(
        controller: keywordController,
        onChanged: searchHome,
        filterIcon: widget.filterIcon,
        goToFilterView: wSearchedList.goToFilterView,
        inputDecoration: widget.inputDecoration,
      ),
      backgroundColor: Colors.white,
    );
  }
}

class _SearchTextField extends StatefulWidget {
  const _SearchTextField({
    required this.controller,
    required this.onChanged,
    required this.goToFilterView,
    required this.filterIcon,
    this.inputDecoration
  });

  final TextEditingController controller;
  final ValueChanged<String> onChanged;
  final VoidCallback goToFilterView;
  final Widget filterIcon;
  final InputDecoration? inputDecoration;

  @override
  _SearchTextFieldState createState() => _SearchTextFieldState();
}

class _SearchTextFieldState extends State<_SearchTextField> {
  bool isShowCloseButton = false;
  @override
  Widget build(BuildContext context) {
    return Row(
      children: [
        const SizedBox(width: 16),
        Expanded(
          child: TextField(
            autofocus: true,
            controller: widget.controller,
            onChanged: (value) {
              widget.onChanged(value);
              if (isShowCloseButton && value.isEmpty) {
                setState(() {
                  isShowCloseButton = false;
                });
              } else if (!isShowCloseButton && value.isNotEmpty) {
                setState(() {
                  isShowCloseButton = true;
                });
              }
            },
            decoration: widget.inputDecoration ?? InputDecoration(
              border: InputBorder.none,
              hintText: Message.getMessage('HOME_Search_Tim_Kiem'),
            ),
          ),
        ),
        if (isShowCloseButton)
          IconButton(
            padding: const EdgeInsets.all(4.0),
            constraints: const BoxConstraints(),
            onPressed: () {
              widget.controller.clear();
              widget.onChanged('');
              setState(() {
                isShowCloseButton = false;
              });
            },
            icon: const Icon(
              Icons.close,
              color: Color(0xff2a367e),
            ),
            splashRadius: 24,
          ),
        IconButton(
          padding: EdgeInsets.zero,
          onPressed: () {
            FocusScope.of(context).requestFocus(FocusNode());
            widget.goToFilterView();
          },
          icon: Image.network(
            'https://8xland.com/images/Upload/tblHinhAnh/2213/icon_filter.png',
            width: 28,
            height: 28,
          ),
          splashRadius: 24,
        ),
      ],
    );
  }
}
