library wow_cache_manager;

import 'dart:io';

import 'package:flutter_cache_manager/flutter_cache_manager.dart';

part 'cache_manager/base_cache_manager.dart';
part 'cache_manager/image_cache_manager.dart';
part 'cache_manager/file_cache_manager.dart';
part 'cache_manager/video_cache_manager.dart';

class WowCacheManager {
  WowCacheManager._();

  factory WowCacheManager() => _instance;

  static final WowCacheManager _instance = WowCacheManager._();

  /// Lấy file
  ///
  /// Nếu ko có file dưới cache thì tự động lưu cache
  static Future<File> getFile(String url) {
    return _FileCacheManager().getCacheFile(url);
  }

  /// Xoá file cache
  static Future<void> removeFile(String url) {
    return _FileCacheManager().removeFile(url);
  }

  /// Kiểm tra xem File có lưu cache chưa
  static Future<bool> isFileCached(String url) {
    return _FileCacheManager().isCached(url);
  }

  /// Lưu file xuống cache
  static Future<File> saveFileToCache(String url) {
    return _FileCacheManager().saveToCache(url);
  }

  /// Xoá tất cả cache file
  static Future<void> emptyCacheFile() {
    return _FileCacheManager().emptyCache();
  }

  /// Lấy ảnh
  ///
  /// Nếu không có ảnh dưới cache thì tự động lưu cache
  static Future<File> getImage(String url) {
    return _ImageCacheManager().getCacheFile(url);
  }

  /// Xoá ảnh cache
  static Future<void> removeImage(String url) {
    return _ImageCacheManager().removeFile(url);
  }

  /// Kiểm tra xem ảnh đã lưu cache chưa
  static Future<bool> isImageCached(String url) {
    return _ImageCacheManager().isCached(url);
  }

  /// Lưu ảnh xuống cache
  static Future<File> saveImageToCache(String url) {
    return _ImageCacheManager().saveToCache(url);
  }

  /// Xoá tất cả cache hình ảnh
  static Future<void> emptyCacheImage() {
    return _ImageCacheManager().emptyCache();
  }

  /// Lấy video
  ///
  /// Nếu không có video dưới cache thì tự động lưu cache
  static Future<File> getVideo(String url) {
    return _VideoCacheManager().getCacheFile(url);
  }

  /// Xoá video cache
  static Future<void> removeVideo(String url) {
    return _VideoCacheManager().removeFile(url);
  }

  /// Kiểm tra video có lưu cache chưa
  static Future<bool> isVideoCached(String url) {
    return _VideoCacheManager().isCached(url);
  }

  /// Lưu video xuống cache
  static Future<File> saveVideoToCache(String url) {
    return _VideoCacheManager().saveToCache(url);
  }

  /// Xoá tất cả cache video
  static Future<void> emptyCacheVideo() {
    return _VideoCacheManager().emptyCache();
  }

  /// Xoá tất cả cache
  static Future<void> emptyCacheAll() {
    return Future.wait(
      [
        emptyCacheFile(),
        emptyCacheImage(),
        emptyCacheVideo(),
      ],
    );
  }
}
