import 'package:flutter/material.dart';
import 'package:wowit/methods.dart';

class WowImage extends StatefulWidget {
  /// Data hình ảnh
  final Map<String, Object> dataImage;

  /// field name hình ảnh [fieldName]!= ['hinhAnh']
  final String? fieldName;

  final double? width;
  final double? height;

  final Widget? errorBuilder;

  final BoxFit boxFit;

  final Widget? loaddingBuilder;

  final double scale;

  const WowImage(this.dataImage,{
    this.fieldName,
    this.height,
    this.width,
    this.errorBuilder,
    this.boxFit = BoxFit.cover,
    this.loaddingBuilder,
    this.scale = 1.0,
    Key? key,
  }) : super(key: key);

  @override
  State<WowImage> createState() => _WowImageState();
}

class _WowImageState extends State<WowImage> {
  String? thumbnailUrl;
  String? imageUrl;

  @override
  void initState() {
    thumbnailUrl = Methods.getString(
        widget.dataImage,
        widget.fieldName != null
            ? widget.fieldName! + '_small_path'
            : 'hinhAnh_small_path');

    imageUrl = Methods.getString(
        widget.dataImage,
        widget.fieldName != null
            ? widget.fieldName! + '_image_path'
            : 'hinhAnh_image_path');
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return _imageNetWork(
      Methods.getURL(imageUrl!),
      frameBuilder: (context, child,frame,loadingProgress) {
        if (frame == null) {
          return _imageNetWork(
            Methods.getURL(thumbnailUrl!),
            color: const Color.fromARGB(126, 180, 179, 179),
            blendMode: BlendMode.modulate
          );
        }
        return child;
      },
    );
  }

  Widget _imageNetWork(String src,
      {Widget? errorBuider,
      Color? color,
      BlendMode? blendMode,
      Widget Function(BuildContext, Widget, int?, bool)? frameBuilder,

      }) {
    return Image.network(
      src,
      color: color,
      scale: widget.scale,
      colorBlendMode: blendMode,
      errorBuilder: ((context, error, stackTrace) =>
          errorBuider ?? errorImage()),
      width: widget.width ?? double.infinity,
      height: widget.height ?? double.infinity,
      fit: widget.boxFit,
      frameBuilder:frameBuilder?? ((context, child, frame, wasSynchronouslyLoaded) {
        if (frame != null) {
          return child;
        }
        return errorImage();
      }),
    );
  }

  Widget errorImage({Color? color, BlendMode? blendMode}) {
    return Image.asset(
      'assets/images/default.jpg',
      width: widget.width ?? double.infinity,
      height: widget.height ?? double.infinity,
      fit: widget.boxFit,
    );
  }
}
